#include <alloca.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include <stdlib.h>

typedef struct {
  double x;
  double y;
} point;

double sqr(double x)
{
  return x*x;
}

double dist(point cities[], int i, int j) {
  return sqrt(sqr(cities[i].x-cities[j].x)+
	      sqr(cities[i].y-cities[j].y));
}

double DistSqrd(point cities[], int i, int j) {
  return (sqr(cities[i].x-cities[j].x)+
	  sqr(cities[i].y-cities[j].y));
}

void swap(int *p, int *q)
{
  int tmp=*p;
  *p = *q;
  *q = tmp;
}

void tsp(point cities[], int tour[], int ncities)
{
  int i,j;
  int ThisPt, ClosePt=0;
  double CloseDist;
  int endtour=0;
  
  for (i=1; i<ncities; i++)
    tour[i]=i-1;
  ThisPt = ncities-1;
  tour[0] = ncities-1;
  
  for (i=1; i<ncities; i++) {
    double ThisX = cities[ThisPt].x;
    double ThisY = cities[ThisPt].y;
    CloseDist = DBL_MAX;
    for (j=i; j<ncities; j++) {
      double ThisDist = sqr(cities[tour[j]].x-ThisX);
      if (ThisDist < CloseDist) {
	ThisDist += sqr(cities[tour[j]].y-ThisY);
	if (ThisDist < CloseDist) {
	  CloseDist = ThisDist;
	  ClosePt = j;
	}
      }
    }
    ThisPt = tour[ClosePt];
    swap(&tour[i],&tour[ClosePt]);
  }
}

int main(int argc, char *argv[])
{
  int i, ncities;
  point *cities;
  int *tour;
  FILE *psfile;
  double sumdist = 0.0;

  if (argc!=2) {
    fprintf(stderr, "usage: %s <ncities>", argv[0]);
    exit(1);
  }
  ncities = atoi(argv[1]);
  cities = alloca(ncities*sizeof(point));
  tour = alloca(ncities*sizeof(int));
  for (i=0; i<ncities; i++) {
    cities[i].x = ((double)(random()))/(double)(1U<<31);
    cities[i].y = ((double)(random()))/(double)(1U<<31);
  }
  tsp(cities,tour,ncities);
  psfile = fopen("tsp.eps","w");
  fprintf(psfile, "%%!PS-Adobe-2.0 EPSF-1.2\n%%%%BoundingBox: 0 0 300 300\n");
  fprintf(psfile, "1 setlinejoin\n0 setlinewidth\n");
  fprintf(psfile, "%f %f moveto\n",
	  300.0*cities[tour[0]].x, 300.0*cities[tour[0]].y);
  printf("%d\n", tour[0]);
  for (i=1; i<ncities; i++) {
    printf("%d\n", tour[i]);
    fprintf(psfile, "%f %f lineto\n",
	    300.0*cities[tour[i]].x, 300.0*cities[tour[i]].y);
    sumdist += dist(cities, tour[i-1], tour[i]);
  }
  fprintf(psfile,"stroke\n");
  printf("sumdist = %f\n", sumdist);
  exit(0);
}
