#include <alloca.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include <stdlib.h>

typedef struct {
  double x;
  double y;
} point;

double sqr(double x)
{
  return x*x;
}

double dist(point cities[], int i, int j) {
  return sqrt(sqr(cities[i].x-cities[j].x)+
	      sqr(cities[i].y-cities[j].y));
}

double DistSqrd(point cities[], int i, int j) {
  return (sqr(cities[i].x-cities[j].x)+
	  sqr(cities[i].y-cities[j].y));
}

void swap(point *p, point *q)
{
  point tmp=*p;
  *p = *q;
  *q = tmp;
}

void tsp(point cities[], point tour[], int ncities)
{
  int i,j;
  int ClosePt=0;
  double CloseDist;
  int endtour=0;
  
  for (i=1; i<ncities; i++)
    tour[i]=cities[i-1];
  tour[0] = cities[ncities-1];
  
  for (i=1; i<ncities; i++) {
    double ThisX = tour[i-1].x;
    double ThisY = tour[i-1].y;
    CloseDist = DBL_MAX;
    for (j=ncities-1; ;j--) {
      double ThisDist = sqr(tour[j].x-ThisX);
      if (ThisDist <= CloseDist) {
	ThisDist += sqr(tour[j].y-ThisY);
	if (ThisDist <= CloseDist) {
	  if (j < i)
	    break;
	  CloseDist = ThisDist;
	  ClosePt = j;
	}
      }
    }
    swap(&tour[i],&tour[ClosePt]);
  }
}

int main(int argc, char *argv[])
{
  int i, ncities;
  point *cities;
  point *tour;
  FILE *psfile;
  double sumdist = 0.0;

  if (argc!=2) {
    fprintf(stderr, "usage: %s <ncities>", argv[0]);
    exit(1);
  }
  ncities = atoi(argv[1]);
  cities = alloca(ncities*sizeof(point));
  tour = alloca(ncities*sizeof(point));
  for (i=0; i<ncities; i++) {
    cities[i].x = ((double)(random()))/(double)(1U<<31);
    cities[i].y = ((double)(random()))/(double)(1U<<31);
  }
  tsp(cities,tour,ncities);
  psfile = fopen("tsp.eps","w");
  fprintf(psfile, "%%!PS-Adobe-2.0 EPSF-1.2\n%%%%BoundingBox: 0 0 300 300\n");
  fprintf(psfile, "1 setlinejoin\n0 setlinewidth\n");
  fprintf(psfile, "%f %f moveto\n",
	  300.0*tour[0].x, 300.0*tour[0].y);
  for (i=1; i<ncities; i++) {
    fprintf(psfile, "%f %f lineto\n",
	    300.0*tour[i].x, 300.0*tour[i].y);
    sumdist += dist(tour, i-1, i);
  }
  fprintf(psfile,"stroke\n");
  printf("sumdist = %f\n", sumdist);
  exit(0);
}
